##########################################
#Author: Kishan Silva Ratnayake (ratki01)
#Created on: 10.16.2013
#Updated on: 10.16.2013
#
##########################################

use strict;
use warnings;
use REST::Client;
use Path::Class;
use autodie; # die if problem reading or writing a file

if(scalar(@ARGV) < 2)
{
        
        print "\nUsage: perl bulk_mf_vc_import.pl <DA_HOST>:<PORT> <PATH_TO_XML>\n";
        print "\n<DA_HOST> - Data Aggregator hostname/ip\n";
        print "<PORT> - Data Aggregator port - default is 8581\n";
        print "<PATH_TO_XML> - Directory path to the location of the MF and VC XMLs\n";
        print "\nExample:\n";
        print "perl bulk_mf_vc_import.pl IM-DA:8581 C:\\dev\\custom \n\n";

        print "**Also make sure you have the following directories created under the path provided:\n";
        print "  1. Directory called 'vc' -- directory containing all the Vendor Certificate XMLs you want to import. \n";
        print "  2. Directory called 'mf' -- directory containing all Metric Family XMLs you want to import. \n";
        exit 1;
}

my $mydir = $ARGV[1];
my $DA_URL = $ARGV[0];
my $vc_dir = $mydir . '\vc';
my $mf_dir = $mydir . '\mf';

if(!-d $mydir)
{
    print "'$mydir' DOES NOT EXIST - please specify a path that exists.\n";
    exit 1;
}

#make connection to DA web service
my $client = REST::Client->new();
$client->setHost('http://'. $DA_URL);
$client->addHeader('Accept', 'application/xml');

if(-d $mf_dir)
{
    my $dir = dir($mf_dir);
    open (MYFILE, '>'.$mydir.'\MetricFamily_creation_output.txt');
    
    #open dir and get list of files
    opendir(D, "$mf_dir") || die "Can't opedir $mf_dir: $!\n";
    my @list = grep !/^\.\.?$/, readdir(D);
    closedir(D);

    #open each file, read contents and POST Vendor Cert to DA.
    my $fileopen;
    my $content;
    my $x=0;
    my $success = 0;
    my $unsuccess = 0;
    my $response_details;
    for($x=0;$x<scalar(@list);$x++)
    {
        my $file = $list[$x];
        
        $fileopen = $dir->file($file);
        $content = $fileopen->slurp();
        print MYFILE "\n\nCreating Metric Family from $file ... ";    
        $client->POST('/typecatalog/metricfamilies', $content);
        if($client->responseCode() eq '200')
        {
                #print MYFILE "Successfully created \n\n\n";
                #print MYFILE "=========================================================\n\n";
                $success++;
        }
        else
        {
            $response_details = $response_details . "\n\nCreating Metric Family from $file ... ";
            $response_details = $response_details . '**UNSUCCESSFUL**' . "\n\n";
            $response_details = $response_details . "<Responsecode>".$client->responseCode()."</Responsecode>\n\n";
            $response_details = $response_details . "<ResponseContent>\n". $client->responseContent(). "</ResponseContent>\n\n";
            $response_details = $response_details . "=========================================================\n\n";
            $unsuccess++;
        }
    }
    if($x==0)
    {
        print MYFILE "There were no Metric Families to import.\n";
        print "\n**There were no Metric Families to import.\n";
    }
    else
    {
            print "\nPlease view the following file for details on Metric Family imports: $mydir\\MetricFamily_creation_output.txt \n";
    }
    print MYFILE "Summary of Metric Family import\n";
    print MYFILE "===============================\n\n";
    print MYFILE "Total number of Metric Families: " . scalar(@list) . "\n";
    print MYFILE "Number of successful imports: " . $success . "\n";
    print MYFILE "Number of unsucessful imports: " . $unsuccess . "\n";
    
    if($unsuccess > 0)
    {
            print MYFILE "\n\n";
            print MYFILE "Details of unsuccessful Metric Family imports:\n";
            print MYFILE "===================================================\n";
            print MYFILE $response_details;
    }
    close (MYFILE);   
    
}

if(-d $vc_dir)
{
    
    open (MYFILE, '>'.$mydir.'\VendorCert_creation_output.txt');
 
    my $dir = dir($vc_dir);

    #open dir and get list of files
    opendir(D, "$vc_dir") || die "Can't opedir $vc_dir: $!\n";
    my @list = grep !/^\.\.?$/, readdir(D);
    closedir(D);


    #open each file, read contents and POST Vendor Cert to DA.
    my $fileopen;
    my $content;
    my $x=0;
    my $success = 0;
    my $unsuccess = 0;
    my $response_details;
    for($x=0;$x<scalar(@list);$x++)
    {
        my $file = $list[$x];
        
        $fileopen = $dir->file($file);
        $content = $fileopen->slurp();
        
        $client->POST('/typecatalog/certifications/snmp', $content);
        if($client->responseCode() eq '200')
        {
                #print MYFILE "Successfully created \n\n\n";
                #print MYFILE "=========================================================\n\n";
                $success++; 
        }
        else
        {
            $response_details = $response_details . "\n\nCreating Vendor Certification from $file ... ";
            $response_details = $response_details . '**UNSUCCESSFUL**' . "\n\n";
            $response_details = $response_details . "<Responsecode>".$client->responseCode()."</Responsecode>\n\n";
            $response_details = $response_details . "<ResponseContent>\n". $client->responseContent(). "</ResponseContent>\n\n";
            $response_details = $response_details . "=========================================================\n\n";
            $unsuccess++; 
        }
    }
    if($x==0)
    {
        print MYFILE "There were no Vendor Certs to import.\n";
        print "\n**There were no Vendor Certs to import.\n";
    }
    else
    {
        print "\nPlease view the following file for details on Vendor Cert imports: $mydir\\VendorCert_creation_output.txt \n";        
    }
    print MYFILE "Summary of Vendor Certificate import\n";
    print MYFILE "====================================\n\n";
    print MYFILE "Total number of vendor certificates: " . scalar(@list) . "\n";
    print MYFILE "Number of successful imports: " . $success . "\n";
    print MYFILE "Number of unsucessful imports: " . $unsuccess . "\n";
    
    if($unsuccess > 0)
    {
            print MYFILE "\n\n";
            print MYFILE "Details of unsuccessful Vendor Certificate imports:\n";
            print MYFILE "===================================================\n";
            print MYFILE $response_details;
    }
    
    close (MYFILE);

}